import React from 'react';
import { Modal, ModalProps, Input, Button, message } from 'antd';
import styles from './FolderSetModal.less';
import classNames from 'classnames';
import { useCallback } from 'react';
import system from '@/services/system';
import { useState } from 'react';
import { useEffect } from 'react';
import { fetchLocalApi } from '@/utils/request';
import { handleRequest } from '@/utils/tool';

export default function FolderSetModal(props: ModalProps) {
  const { className, ...restProps } = props;
  const [folderPath, setFolderPath] = useState('');
  const [loading, setLoading] = useState(false);

  const onSetFolderPos = useCallback(async () => {
    const nextPath = await system.chooseFolder();
    console.log(nextPath);
    if (nextPath) {
      setFolderPath(nextPath);
    }
  }, []);

  useEffect(() => {
    (async () => {
      const res = await fetchLocalApi<string>('getWorkSpace');
      handleRequest(res).success(() => {
        setFolderPath(res.data || '');
      });
    })();
  }, []);

  const updateFolderPath = useCallback(async () => {
    if (!folderPath) {
      message.error('请选择文件夹');
      return;
    }
    setLoading(true);
    const res = await fetchLocalApi<null>('setWorkSpace', { path: folderPath });
    setLoading(false);
    handleRequest(res).success(() => {
      if (restProps.onCancel) {
        restProps.onCancel();
      }
    });
  }, [folderPath]);

  return (
    <Modal
      title="同步设置"
      className={classNames(styles.modal, className)}
      width={360}
      {...restProps}
      footer={null}
      maskClosable={false}
    >
      <span className={styles.label}>「工作空间」本地盘位置</span>
      <Input
        value={folderPath}
        size="small"
        suffix={
          <Button type="link" onClick={onSetFolderPos}>
            选择
          </Button>
        }
      />

      <div className={styles.btnGroup}>
        <Button
          loading={loading}
          className={styles.btn}
          type="primary"
          onClick={updateFolderPath}
        >
          更新资料
        </Button>
      </div>
    </Modal>
  );
}
