import system from '@/services/system';
import { identity } from 'lodash';
import { isClient } from './tool';

const keepSaveKeyMap = new Map();

const persistKeyName = '_persistKeys'; // key1,key2

const storage = {
  get(key: string) {
    return localStorage.getItem(key);
  },
  set(key: string, value: string, keep = false) {
    if (keep) {
      system.storage.set(key, value);
      // keepSaveKeyMap.set(key, true);
    }
    localStorage.setItem(key, value);
  },
  remove(key: string) {
    system.storage.remove(key);
    keepSaveKeyMap.delete(key);
    return localStorage.removeItem(key);
  },
  clear() {
    // 保留keep过的属性
    const tempStorage: { [key: string]: any } = {};
    keepSaveKeyMap.forEach((_, key) => {
      tempStorage[key] = storage.get(key);
    });
    localStorage.clear();
    keepSaveKeyMap.forEach((_, key) => {
      localStorage.setItem(key, tempStorage[key]);
    });
  },
};

export function initialStorage() {
  if (isClient && window.addIpcRendererListener) {
    storage.clear();
    window.addIpcRendererListener(
      'initialStorageData',
      (_, hash: { keyList?: string[]; [key: string]: any }) => {
        const kList = hash.keyList || [];
        kList.forEach((key: string) => {
          keepSaveKeyMap.set(key, true);
          storage.set(key, hash[key]);
        });
      },
    );
  }
}

export default storage;
